# TODO: Main class for consuming SDMX web services
# 
# Author: E922480
###############################################################################

# uncomment for local usage
#library(rJava) 
#library(zoo) 
#library(tcltk)
# ux  
#.jinit(classpath="lib/SDMX.jar:lib/SDMX_SEC.jar") 
# win 
#.jinit(classpath="lib/SDMX.jar;lib/SDMX_SEC.jar") 
#source("TSConverter.R"); 
# get the key family of the given dataflow (after retrieving the key family)

# get the list of dataflows of this provider
getDataFlows <- function(client, pattern='') {		
	res <- .jcall(client,"Ljava/util/Map;","getFlows", pattern)
	list = convertHashTable(res)
	return(list)
}

# get the list of key families of this provider
getKeyFamilies <- function(client) {		
	res <- .jcall(client,"Ljava/util/Map;","getKeyFamilies")
	list = convertHashTable(res)
	numOfFamilies = length(list)
	if( numOfFamilies != 0 ) {
		for (i in 1:numOfFamilies)  {
			print(paste(names(list[i]), ' : ', list[[i]]))
		}
	}
	return(list)
}

# get the key family of the given dataflow (after retrieving the key family)
getKeyFamily <- function(client, dataflow) {		
	res <- .jcall(client,"Ljava/lang/String;","getKeyFamily", dataflow, evalString = TRUE)
	return(res)
}

# get dimensions of the given dataflow (after retrieving the key family)
getDimensions <- function(client, dataflow) {		
	res <- .jcall(client,"Ljava/util/List;","getDimensions", dataflow);
	jlist <- .jcall(res,"[Ljava/lang/Object;","toArray");
	list = convertStringList(jlist)
	return(list)
}

# get the time series matching the parameters
getTimeSeries <- function(client, id, start='', end='') {		
	#run getTimeSeries method. 
	res <- .jcall(client,"Ljava/util/List;","getTimeSeriesFromID", id, start, end, use.true.class=TRUE)
	#convert to an R list
	res = convertTSList(res)
	return(res) 
}

# get the time series matching the parameters
getTimeSeriesFromHandler <- function(provider, id, start='', end='') {  	
  
  handler <- initHandler(provider)
  if(!is.null(handler)){
    #run getTimeSeries method. 
    res <- .jcall(handler,"Ljava/util/List;","getTimeSeriesFromID", provider, id, start, end, use.true.class=TRUE)
  }
  #convert to an R list
  res = convertTSList(res)
  return(res) 
}

# get the time series matching the parameters
getSDMX <- function(provider, id, start='', end='') {
  return(getTimeSeriesFromHandler(provider, id, start, end))
}

# get the list of available data providers
getDataProviders <- function() {		
	jlist=J("it.bankitalia.reri.sia.sdmxws.api.SDMXClientFactory")$getAllProviders()
	list = convertStringList(jlist)
	return(list) 
}

# create client
SdmxClient <- function(provider){
	client <- J("it.bankitalia.reri.sia.sdmxws.api.SDMXClientFactory")$createClient(provider, 'R')
	if(!is.null(client)){
		if(.jcall(client,"Z", "needsCredentials")) {
			if(inherits(try(account<-get_user_password_tcl(), silent = TRUE), "try-error"))  account<-get_user_password()
			.jcall(client,"V", "setCredentials", account[1], account[2])
		}
	}
	return(client)
}

# create client
initHandler <- function(provider){
  handler <- J("it.bankitalia.reri.sia.sdmxws.api.SdmxClientHandler")$getInstance()
  if(!is.null(handler)){
    if(.jcall(handler,"Z", "needsCredentials", provider)) {
      if(inherits(try(account<-get_user_password_tcl(), silent = TRUE), "try-error"))  account<-get_user_password()
      .jcall(handler,"V", "setCredentials", provider, account[1], account[2])
    }
  }
  return(handler)
}

get_user_password <- function() {
  cat("user: ")
  b <- readline()
	cat("Password: ")
	system("stty -echo")
	a <- readline()
	system("stty echo")
	cat("\n")
	ret = b;
  ret[2] = a;
	return(ret);
}

get_user_password_tcl<-function(){
	wnd<-tktoplevel();tclVar("")->passVar;tclVar("")->userVar;
	#Label  
	tkgrid(tklabel(wnd,text="Enter user and password:"));
	#User box  
	tkgrid(tkentry(wnd,textvariable=userVar)->userBox);
	#Password box  
	tkgrid(tkentry(wnd,textvariable=passVar,show="*")->passBox);
	#Hitting return will also submit password  
	tkbind(passBox,"<Return>",function() tkdestroy(wnd));
	#OK button  
	tkgrid(tkbutton(wnd,text="OK",command=function() tkdestroy(wnd)));
	#Wait for user to click OK  
	tkwait.window(wnd);
	password<-tclvalue(passVar);
	user<-tclvalue(userVar);
	ret = user;
	ret[2] = password;
	return(ret);
}

SDMXtest<-function(){
	client = SdmxClient('ECB')
	getKeyFamily(client, 'EXR')
	getDimensions(client, 'EXR')
	ans=getTimeSeries(client, 'EXR.M.GBP.EUR.SP00.A')
	plot(ans[[1]])

	client = SdmxClient('ECB_SECURE')
	getKeyFamily(client, 'EXR')
	getDimensions(client, 'EXR')
	ans=getTimeSeries(client, 'EXR.M.GBP.EUR.SP00.A')
	plot(ans[[1]])

	client = SdmxClient('OECD')
	getKeyFamily(client, 'REFSERIES')
	getDimensions(client, 'REFSERIES')
	ans=getTimeSeries(client, 'REFSERIES.AUS.PPPGDP.A')
	plot(ans[[1]])

	client = SdmxClient('ISTAT')
	getKeyFamily(client, '115_200_Q')
	getDimensions(client, '115_200_Q')
	ans=getTimeSeries(client, '115_197.*.IT.*.*.0020.2005.*')
	plot(ans[[1]])

}
